#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 08/28/2001
#################################################################################
#
# Configure Puppet - Script part of Webchat designed to configure Puppets
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error Logs
use strict;							# Strict variable enforcement

use Image::Grab;						# Image grabber
use Image::Size;						# Image sizer
use Digest::MD5 qw(md5_hex);					# MD5 digest cryptographic algorithm

use Ethereal::Database;						# Database handler
use Ethereal::Filter;						# Filter system
use Ethereal::Login;						# Login functionality
use Ethereal::Option;						# Options handling
use Ethereal::Param;						# Parameter control
use Ethereal::Table;						# Table control


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter handle

my %config;							# Configuration tag

#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();
	$param    = new Ethereal::Param();

	# Connect and fetch
	$database->Connect();
	$param->GetParam($cgi);

	# Fetch hash
	$database->GetHashConfiguration(\%config);


	# Document header
	print $cgi->header();
	$database->DocumentGetHeader();

	# Login handle
	$login = new Ethereal::Login($database, $cgi, $param);

	# Authenthication
	if ($login->GetVerificationNormal())
	{
		# Print title
		print "$config{'TagTitle'}\n<BR>\n";
	
		# For additions and removal
		unless  ((defined($param->{$config{'TxtSubmitView'}})) 
                     || (defined($param->{$config{'TxtSubmitButton'}})))
		{ InitialDisplay($database, $cgi, $param, \%config); }

		# For puppet changes and viewing
		else
		{ PuppetControl($database, $cgi, $param, \%config); }
	}

	# Document footer	
	$database->DocumentGetFooter();


#################################################################################
# Sub-Routines
#################################################################################

#####################
# ImageSize
#
# Downloads a picture off the net and determins it's size as to be able place 
# the defined sizes on the database

sub ImageSize
{
        ####################
        # Data Members

	my $database = shift;					# Database handle	
        my $link     = shift;					# URL to picture

	my $image;						# Image handle
        my $name;						# Name of file

	my %system;						# System hash
	my @sizes;						# Size returned


        ####################
        # Program Area

	# Link to Hash
	$database->GetHashSystem(\%system);

        # Initialize handles
        $image = new Image::Grab;

        # Generate random name
        $name  = md5_hex(rand());;
	$name  = "$system{'LocTemp'}/$name";

        # Retreive image
        $image->url($link);
        $image->grab;

        # Write to file
        open(IMAGE, ">$name") || die("Cannot open $name : $!");
        print IMAGE $image->image;
        close(IMAGE);

	# Determine image size and remove file
	@sizes = imgsize("$name");
	unlink("$name");

	return @sizes;						# Return values
}


#####################
# InitialDisplay
#
# Handles the general output of the initial display screen.  Which will allow one
# to change password, E-Mail addresses and/or passwords.

sub InitialDisplay						# Database, CGI, Param
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $config   = shift;					# Config hash

	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $filter   = new Ethereal::Filter();			# Filter handle
	my $table    = new Ethereal::Table($cgi);		# Table handle

	my @puppets;						# Array of puppets

	my $list;						# List of puppets
	my $buttonview;						# Button view
	my $buttonrem;						# Button remove
	my $buttons;						# Buttons used

	my $choice;						# Choice of the matter
	my $message;						# Display message


	#####################
	# Program area	
	
	# Addition operation
	if ( defined($param->{$config->{'TxtSubmitAddition'}}) ) 
	{
		# Strip HTML
		$param->{'PUPADD'} = $filter->StripHTML($param->{'PUPADD'});

		# Additional filtering
		$param->{'PUPADD'} =~ s/~//gs;


		# Retreive information
		($choice) = $database->DataGet('SELECT PuppetName 
			FROM  Puppet 
			WHERE PuppetName=?', $param->{'PUPADD'});

		# Verification
		if ((length($param->{'PUPADD'}) > 2) && ($param->{'PUPADD'} !~ /^\s/) && (!defined($choice)))
		{
			# Insertion upon acceptance
			$data->do("INSERT INTO Puppet 
				(PuppetName, 
                                 PuppeteerLogin,
				 PuppetTextSize)
				VALUES(?,?,?)", {},
			 $param->{'PUPADD'}, $param->{'USER'}, 3);

			# Display status
			$message = $config->{'MsgCreate'};
		}
		else
		{ $message = $config->{'MsgExists'}; }
	}

	# Removal operation
	if (defined($param->{$config->{'TxtSubmitDelete'}}))
	{
		# Retreive information
		($choice) = $database->DataGet('SELECT PuppetName
			FROM Puppet 
			WHERE PuppetName=? 
			  AND PuppeteerLogin=?', $param->{'PUPPET'}, $param->{'USER'});

		# Verification
		if (defined($choice))
		{
			# Removal of secondary tables
			$data->do("DELETE FROM PuppetInvitation WHERE PuppetName=?", {}, $param->{'PUPPET'} ); 
			$data->do("DELETE FROM PuppetIgnore WHERE PuppetName=?",     {}, $param->{'PUPPET'} ); 

			# Removal of primary tables
			$data->do("DELETE FROM Puppet WHERE PuppetName=?",           {}, $param->{'PUPPET'} );

			# Message of success
			$message = $config->{'MsgDeletePass'}; 
		}		
		else
		{ $message = $config->{'MsgDeleteFail'}; }
	}


	# Cleanup
	$param->Cleanup(
		$config->{'TxtSubmitAddition'},
		$config->{'TxtSubmitDelete'});


	# Display of values
	# Retreive a list of puppets
	$database->GetListPuppetYours(\@puppets, $param->{'USER'});


	# Prepare widgets
	$list       = $cgi->scrolling_list('PUPPET', \@puppets, $puppets[0], 10);
	$buttonview = $cgi->submit($config->{'TxtSubmitView'});
	$buttonrem  = $cgi->submit($config->{'TxtSubmitDelete'});
	$buttons    = "$buttonview<BR><BR>$buttonrem";

	# Selection/removal
	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}), "\n";


	# Table top
	$table->MakeTop();

	# Status message
	if (defined($message)) 
	{ 
		$table->MakeSingle($message); 
		$table->MakeBlank(); 
	}
	
	# Select puppet
	$table->MakeSingle($config->{'TagPupDelSelect'});
	$table->MakeValid($list, $buttons);
	$table->MakeBlank();

	# Select kit
	$table->MakeSingle($config->{'TagPupAddition'});
	$table->MakeValid($cgi->textfield('PUPADD', '', 20, 30), $cgi->submit($config->{'TxtSubmitAddition'}));
	$table->MakeBottom();

	# End form
	print $cgi->end_form();
}


#####################
# PuppetControl
#
# Sub-routine that will handle the change of information for a puppet.  This
# is assuming that the submit button was pressed.

sub PuppetControl
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $config   = shift;					# Config hash

	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $option   = new Ethereal::Option();			# Option handle
	my $table    = new Ethereal::Table($cgi);		# Table handle

	my $statement;						# Query handle

	my $choice   = 0;					# Choice of the matter

	my $admin;						# Administrative access
	my $mod;						# Moderator access
	my $super;						# Supervisory access

	my $info;						# Puppet information
	my $gummies;						# Gummies html
	my $gummy;						# Default entry
	my $optgummy;						# Optgummy

	my $inline;						# Parameters
	my $url      = $cgi->url();				# Self reference

	my @gummies;						# List of gummies


	#####################
	# Program area

	# Change of information and verify for appripriate button press
	if ( defined($param->{$config->{'TxtSubmitButton'}}) )
	{ 
		PuppetControlWrite($database, $param, $config); 

		# No repetition
		$choice = 1;
	}


	# Data Retreival
	# Determin privileges
	($super, $admin, $mod) = $database->DataGetPrivs($param->{'USER'});


	# Back button
	# Generate Inline
	$inline = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'});	

	# Display
	$table->MakeTop();
	$table->MakeSingle("<A HREF=\"$url$inline\">$config->{TagBack}</A>");
	$table->MakeBottom();


	# Gather information
	# Prepare and execute
	$statement = $data->prepare("SELECT PuppetName,
			 PuppetTag,
			 PuppetPic,
			 PuppetPicHeight,
			 PuppetPicWidth,
			 PuppetTitle,
			 PuppetBio,
			 PuppetGummy,
			 PuppetTextColour,
			 PuppetTextSize,
			 PuppetActive
			FROM  Puppet
			WHERE PuppetName=? 
			  AND PuppeteerLogin=?");
	$statement->execute($param->{'PUPPET'}, $param->{'USER'});

	# Retreive into hash
	$info = $statement->fetchrow_hashref();

	# Gummy selection list
	if ($choice != 1)
	{
		# Retreive Icons
		$config->{'OptGummy'} = ($mod   eq 'yes') ? "$config->{OptGummy}:$config->{OptGummyMod}" : "$config->{OptGummy}";
		$config->{'OptGummy'} = ($super eq 'yes') ? "$config->{OptGummy}:$config->{OptGummySup}" : "$config->{OptGummy}";
		$config->{'OptGummy'} = ($admin eq 'yes') ? "$config->{OptGummy}:$config->{OptGummyAdm}" : "$config->{OptGummy}";

		# Retreive Files
		$config->{'OptGummyFiles'} = ($mod   eq 'yes') ? "$config->{OptGummyFiles}:$config->{OptGummyModFiles}" : "$config->{OptGummyFiles}";
		$config->{'OptGummyFiles'} = ($super eq 'yes') ? "$config->{OptGummyFiles}:$config->{OptGummySupFiles}" : "$config->{OptGummyFiles}";
		$config->{'OptGummyFiles'} = ($admin eq 'yes') ? "$config->{OptGummyFiles}:$config->{OptGummyAdmFiles}" : "$config->{OptGummyFiles}";
	}

	# Put into array
	$option->Split(\@gummies, $config->{'OptGummy'});

	# Determin default and format
	$gummy   = $option->Find($config->{'OptGummyFiles'}, $config->{'OptGummy'}, $info->{'PuppetGummy'});
	$gummies = $cgi->popup_menu('GUMMY', \@gummies, $gummy);


	# Add hint for users
	$info->{'PuppetPic'} = (defined($info->{'PuppetPic'})) ? $info->{'PuppetPic'} : 'http://';
	$info->{'PuppetBio'} = (defined($info->{'PuppetBio'})) ? $info->{'PuppetBio'} : 'http://';


	# Display of information
	# Star form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}, 'PUPPET', $param->{'PUPPET'}), "\n";

	# Table top
	$table->MakeTop();

	# Most clueful information
	$table->MakeValid($config->{'TagPupName'}, "<B>$info->{'PuppetName'}</B>");
	$table->MakeBlank();

	# Image information
	$table->MakeValid($config->{'TagPupImageLink'}, $cgi->textfield('PUPIMG', $info->{'PuppetPic'}, 30, 200));
	$table->MakeValid($config->{'TagPupImgWidth'},  "<I>$info->{'PuppetPicWidth'}</I>");
	$table->MakeValid($config->{'TagPupImgHeight'}, "<I>$info->{'PuppetPicHeight'}</I>");
	$table->MakeBlank();

	# Bio Url
	$table->MakeValid($config->{'TagPupBio'}, $cgi->textfield('PUPBIO', $info->{'PuppetBio'}, 30, 200));
	$table->MakeValid($config->{'TagPupGummy'}, $gummies);
	$table->MakeBlank();

	# More complexe tagline
	$table->MakeSingle($config->{'TagPupTagline'});
	$table->MakeSingle("<TEXTAREA NAME=\"PUPTAG\", ROWS=3, COLS=65, WRAP=\"SOFT\">$info->{'PuppetTag'}</TEXTAREA>" );
	$table->MakeBlank();


	# More complexe title
	$table->MakeSingle($config->{'TagPupTitle'});
	$table->MakeSingle("<TEXTAREA NAME=\"PUPTIT\", ROWS=3, COLS=65, WRAP=\"SOFT\">$info->{'PuppetTitle'}</TEXTAREA>");
	$table->MakeBlank();

	# Finish table
	$table->MakeValid($cgi->submit($config->{'TxtSubmitButton'}), $cgi->reset($config->{'TxtResetButton'}));
	$table->MakeBottom();


	# End of form
	print $cgi->end_form();

	# Finish statement
	$statement->finish();
}


#####################
# PuppetControlWrite
#
# Sub-routine that will handle the change of information for a puppet.  This
# is used as a sub process to handle database interaction.

sub PuppetControlWrite
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $param    = shift;					# Parameter handle
	my $config   = shift;					# Config hash

	my $data     = $database->{'HANDLE'};			# Database connection handle

	my $filter   = new Ethereal::Filter();			# Filter handle
	my $option   = new Ethereal::Option();			# Option handle

	my $admin;						# Administrative rights
	my $mod;						# Moderator rights
	my $super;						# Supervisory rights

	my $legal;						# Image legal
	my $height;						# Image height
	my $width;						# Image width

	my $cbio;						# Bio
	my $cgum;						# Gummy
	my $ctag;						# Cleaned tag
	my $ctit;						# Cleaned title


	#####################
	# Program area

	# Data retrieval
	# Determin privileges
	($super, $admin, $mod) = $database->DataGetPrivs($param->{'USER'});


	# Image handling
	# If valid URL
	if (length($param->{'PUPIMG'}) > 10) 
	{
		# Legal verification
		($legal) = $database->DataGetImageBan($param->{'PUPIMG'});

		# If legal
		unless (defined($legal))
		{	
			# Capture and determin size
			($width, $height, $_) = ImageSize($database, $param->{'PUPIMG'}); 
	
			# Update database with new information
			$data->do("UPDATE Puppet
				SET PuppetPic=?, 
				    PuppetPicWidth=?, 
				    PuppetPicHeight=?
				WHERE PuppetName=? 
				  AND PuppeteerLogin=?", {}, 
			 $param->{'PUPIMG'}, 
			 $width, 
			 $height, 
			 $param->{'PUPPET'}, 
			 $param->{'USER'}); 
		}
	}
	else
	{
		# Update database with nulls
		$data->do("UPDATE Puppet 
			SET PuppetPic=NULL, 
		            PuppetPicWidth=NULL, 
			    PuppetPicHeight=NULL
			WHERE PuppetName=? 
			  AND PuppeteerLogin=?", {}, 
		 $param->{'PUPPET'}, 
		 $param->{'USER'}); 
	}

	# Data handling
	# Truncate and clean output
	$ctag = substr($param->{'PUPTAG'}, 0, 500);
	$ctit = substr($param->{'PUPTIT'}, 0, 500);
	$cbio = $param->{'PUPBIO'};

	# Gummy handling
	# Retreive Names
	$config->{'OptGummy'} = ($mod   eq 'yes') ? "$config->{OptGummy}:$config->{OptGummyMod}" : "$config->{OptGummy}";
	$config->{'OptGummy'} = ($super eq 'yes') ? "$config->{OptGummy}:$config->{OptGummySup}" : "$config->{OptGummy}";
	$config->{'OptGummy'} = ($admin eq 'yes') ? "$config->{OptGummy}:$config->{OptGummyAdm}" : "$config->{OptGummy}";

	# Retreive Files
	$config->{'OptGummyFiles'} = ($mod   eq 'yes') ? "$config->{OptGummyFiles}:$config->{OptGummyModFiles}" : "$config->{OptGummyFiles}";
	$config->{'OptGummyFiles'} = ($super eq 'yes') ? "$config->{OptGummyFiles}:$config->{OptGummySupFiles}" : "$config->{OptGummyFiles}";
	$config->{'OptGummyFiles'} = ($admin eq 'yes') ? "$config->{OptGummyFiles}:$config->{OptGummyAdmFiles}" : "$config->{OptGummyFiles}";

	# Determine file
	$cgum = $option->Find($config{'OptGummy'}, $config{'OptGummyFiles'}, $param->{'GUMMY'});	


	# Tag handling
	($ctag) = $filter->SafeHTML($ctag);
	($ctit) = $filter->SafeHTML($ctit);

	# Database update
	# Tagline, Title and Bio link
 	$ctag = (length($ctag) > 3)  ? $ctag : undef;
 	$ctit = (length($ctit) > 3)  ? $ctit : undef;
 	$cbio = ((length($cbio) > 12) && ($cbio =~ /^http:\/\//is)) ? $cbio : undef; 

	# Update fields
	$data->do("UPDATE Puppet 
		SET PuppetTag=?, 
		    PuppetTitle=?,
		    PuppetBio=?,
		    PuppetGummy=? 
		WHERE PuppetName=? 
		  AND PuppeteerLogin=?", {}, 
	 $ctag, 
	 $ctit, 
	 $cbio,
	 $cgum, 
	 $param->{'PUPPET'}, 
	 $param->{'USER'});
}
